using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PlexityHide.GTP;

namespace LetsBuild_WeeklyPlanner_wf
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

    private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
    {
      for (int i=0;i<contextMenuStrip1.Items.Count;i++)
        contextMenuStrip1.Items[i].Enabled=false;
        
      if (contextMenuStrip1.SourceControl == gantt1.TimeItemArea)
      {
        createABookingToolStripMenuItem.Enabled = gantt1.GanttRowFromPoint(gantt1.MouseDownPoint)!=null;
        removeBookingToolStripMenuItem.Enabled = gantt1.FocusedTimeItem != null;
      }

      
      
    }

    private DateTime _start;
    private void Form1_Load(object sender, EventArgs e)
    {
      DoAllTheInitWork(DateTime.Now);
    }

    public void DoAllTheInitWork(DateTime aStartDay)
    {
      DateTime start = aStartDay;
      while (start.DayOfWeek!=DayOfWeek.Sunday)
        start=start.AddDays(-1);
      InitWeekView(start);
      
      // Add some more time item layouts to get color
      gantt1.TimeItemLayouts.GetFromName("yellow").CloneAndAddWithNewName("blue");
      gantt1.TimeItemLayouts.GetFromName("yellow").CloneAndAddWithNewName("green");
      gantt1.TimeItemLayouts.GetFromName("yellow").CloneAndAddWithNewName("black");
      gantt1.TimeItemLayouts.GetFromName("yellow").CloneAndAddWithNewName("purple");
      
      gantt1.TimeItemLayouts.GetFromName("blue").Color=Color.AliceBlue;
      gantt1.TimeItemLayouts.GetFromName("blue").GradientColor = Color.CadetBlue;

      gantt1.TimeItemLayouts.GetFromName("green").Color = Color.GreenYellow;
      gantt1.TimeItemLayouts.GetFromName("green").GradientColor = Color.Green;

      gantt1.TimeItemLayouts.GetFromName("black").Color = Color.DarkGray;
      gantt1.TimeItemLayouts.GetFromName("black").GradientColor = Color.Black;

      gantt1.TimeItemLayouts.GetFromName("purple").Color = Color.Plum;
      gantt1.TimeItemLayouts.GetFromName("purple").GradientColor = Color.Purple;
      
      _rand=new Random();

      // Check if there is a xml file named weekdata if so load it.
      if (System.IO.File.Exists("./weekdata.xml"))
        dataSet1.ReadXml("./weekdata.xml");

    }
    
    
    public Random _rand;
    
    private void InitWeekView(DateTime StartOfWeek)
    {
      _start=StartOfWeek;
      gantt1.DateScaler.LowerBound = _start;
      gantt1.DateScaler.UpperBound = _start.AddDays(1);
      gantt1.DateScaler.StartTime = _start.AddHours(6);
      gantt1.DateScaler.StopTime=_start.AddHours(12);// 12 hours in view
      gantt1.DateScaler.ScrollButtonsVisible=false;

      dataSetDays.Tables["Days"].Rows.Clear();
      for (int i = 0; i < 7; i++)
      {
        DataRow r = dataSetDays.Tables["Days"].NewRow();
        dataSetDays.Tables["Days"].Rows.Add(r);
        r["Name"] = StartOfWeek.DayOfWeek.ToString();
        StartOfWeek = StartOfWeek.AddDays(1);
      }

    }

    private void createABookingToolStripMenuItem_Click(object sender, EventArgs e)
    {
      GanttRow gr=gantt1.GanttRowFromPoint(gantt1.MouseDownPoint);
      if (gr!=null)
      {
        int foreignKeyForResource=(int)(gr.GridNode.ListItemWhenDataBound() as DataRowView)["Id"];

        DataRow r = dataSet1.Tables["Booking"].NewRow();
        r["Text"] = "Note";
        // We are clicking on the time scale equal to Sunday, we have set TimeSpanOffset to 0,-1,-2,-3 etc on the days to 
        // get them to render side by so we can visually compare them in time...
        // When we add new Bookings we must compensate the click with respect to the TimeSpanOffset
        r["Start"]  = gantt1.DateScaler.PixelToTime(gantt1.MouseDownPoint.Y).Subtract(gr.TimeSpanOffset);
        r["Stop"]=((DateTime)r["Start"]).AddHours(1);

        r["Color"]=gantt1.TimeItemLayouts[_rand.Next(0,gantt1.TimeItemLayouts.Count )].Name;
        dataSet1.Tables["Booking"].Rows.Add(r);
      }
    }


    private void gantt1_OnEachListItemWhenDataBound_GridNode(object GTPComponent, EachListItemWhenDataBoundArgs e)
    {
      Layer l = Gantt.GanttRowFromGridNode(e.GTPObject as GridNode).Layers[0];
      if (l.DataSource==null)
      {

        // For each added node we must hook up the Bookings for that day...
        DataView dv = new DataView(dataSet1.Tables["Booking"]);
        DateTime startOfThisDay=_start.AddDays((e.GTPObject as GridNode).Index);
        DateTime stopOfThisDay=_start.AddDays((e.GTPObject as GridNode).Index+1);
        dv.RowFilter = "Start<#"+stopOfThisDay.ToString("yyyy-MM-dd HH:mm")+"# and Stop>#"+startOfThisDay.ToString("yyyy-MM-dd HH:mm")+"#";
        l.Layers.GanttRow.TimeSpanOffset=_start.Subtract(startOfThisDay);
        l.NameInDS_Identity = "Id";
        l.NameInDS_Start = "Start";
        l.NameInDS_Stop = "Stop";
        l.DataSource = dv; 
      }
    }

    private void gantt1_OnEachListItemWhenDataBound_TimeItem(object GTPComponent, EachListItemWhenDataBoundArgs e)
    {
      if ((e.GTPObject as TimeItem).TimeItemTexts.Count==0)
      {
        TimeItemText tit=new TimeItemText();
        tit.Text = (string) (e.CurrencyManagerListItem as DataRowView)["Text"];  
        tit.TimeItemTextLayout=gantt1.TimeItemTextLayouts.GetFromName("default");
        (e.GTPObject as TimeItem).TimeItemTexts.Add(tit);

        tit=new TimeItemText();
        tit.Text = "Booking for "+(string)((e.GTPObject as TimeItem).GanttRow.GridNode.ListItemWhenDataBound() as DataRowView)["Name"];  
        tit.TimeItemTextLayout=gantt1.TimeItemTextLayouts.GetFromName("lower");
        (e.GTPObject as TimeItem).TimeItemTexts.Add(tit);

        (e.GTPObject as TimeItem).TimeItemLayout = gantt1.TimeItemLayouts.GetFromName((string) (e.CurrencyManagerListItem as DataRowView)["Color"]);

      }      
      
    }

    private void gantt1_OnTimeItem_ChangeRow(Gantt aGantt, TimeItemEventArgs e)
    {
      // If we want to allow row changes when here we must understand that a 
      // row change is only a switch of day TimeItem, and the change is on the data...
      e.Allow=false; // No gui operation, its a data operation
      if (e.TimeItem.ListItemWhenDataBound() is DataRowView)
      {
        (e.TimeItem.ListItemWhenDataBound() as DataRowView).BeginEdit();
        TimeSpan ts=e.NewGanttRow.TimeSpanOffset.Subtract(e.TimeItem.GanttRow.TimeSpanOffset);
        (e.TimeItem.ListItemWhenDataBound() as DataRowView)["Start"] = e.TimeItem.Start.Subtract(ts);
        (e.TimeItem.ListItemWhenDataBound() as DataRowView)["Stop"] = e.TimeItem.Stop.Subtract(ts);
        (e.TimeItem.ListItemWhenDataBound() as DataRowView).EndEdit();
      }
    }


    private void removeBookingToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (gantt1.FocusedTimeItem != null)
      {
        (gantt1.FocusedTimeItem.ListItemWhenDataBound() as DataRowView).Delete();
      }

    }

    private void buttonPrevWeek_Click(object sender, EventArgs e)
    {
      InitWeekView(_start.AddDays(-7));
    }

    private void buttonNextWeek_Click(object sender, EventArgs e)
    {
      InitWeekView(_start.AddDays(7));
    }

    private void button1_Click(object sender, EventArgs e)
    {
      dataSet1.WriteXml("./weekdata.xml", XmlWriteMode.IgnoreSchema);

    }
  }
}